<?php

namespace Drupal\active_campaign_api_integration\Plugin;

/**
 * @file
 * Api v3 api calls file.
 */

/**
 * All v3 api callings go through this class.
 *
 * @class Handles all v3 api of ActiveCampaign.
 */
class ApiCaller {

  /**
   * Token variable.
   *
   * @var array
   */
  private array $tokens;

  /**
   * Construct.
   */
  public function __construct() {
    $storage = new StoragesHandlers(\Drupal::database());
    $this->tokens = $storage->getCampaignKeys();
  }

  /**
   * Creating a list on ActiveCampaign.
   *
   * @param string $title
   *   List title.
   *
   * @return string|null
   *   Return list identifies or null.
   */
  public function createList(string $title): string|null {
    $data['stringid'] = str_replace(" ", "-", strtolower($title));
    $data['name'] = $title;
    if (!empty($this->tokens) && $this->tokens[0]['url']) {
      $data['sender_url'] = $this->tokens[0]['url'];

      $site_name = \Drupal::config('system.site')->get('name');
      $site_name = \Drupal::translation()->translate($site_name);

      $data['sender_reminder'] = "You are receiving this because you have sign
       up on $site_name";
      $data['send_last_broadcast'] = FALSE;
      $data['user'] = 1;

      $curl = curl_init();

      curl_setopt_array($curl, [
        CURLOPT_URL => "{$this->tokens[0]['url']}/api/3/lists",
        CURLOPT_RETURNTRANSFER => TRUE,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 30,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "POST",
        CURLOPT_POSTFIELDS => json_encode([
          'list' => $data,
        ]),
        CURLOPT_HTTPHEADER => [
          "Api-Token: {$this->tokens[0]['key']}",
          "accept: application/json",
          "content-type: application/json",
        ],
      ]);

      $response = curl_exec($curl);
      $err = curl_error($curl);
      curl_close($curl);
      if ($err) {
        return NULL;
      }
      $d = json_decode($response, TRUE);
      return $d['list']['id'] ?? NULL;
    }
    return NULL;
  }

  /**
   * Creating fields on ActiveCampaign.
   *
   * @param array $fields
   *   Fields to create.
   *
   * @return array
   *   return ids of fields created.
   */
  public function createFields(array $fields): array {

    $ids = [];
    if ($fields['users']) {
      foreach ($fields['users'] as $field) {
        $data = [
          'title' => $field['label']->__toString(),
          'type' => 'Text Input',
        ];
        $curl = curl_init();
        curl_setopt_array($curl, [
          CURLOPT_URL => "{$this->tokens[0]['url']}/api/3/fields",
          CURLOPT_RETURNTRANSFER => TRUE,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "POST",
          CURLOPT_POSTFIELDS => json_encode([
            'field' => $data,
          ]),
          CURLOPT_HTTPHEADER => [
            "Api-Token: {$this->tokens[0]['key']}",
            "accept: application/json",
            "content-type: application/json",
          ],
        ]);

        $response = curl_exec($curl);
        curl_close($curl);
        $d = json_decode($response, TRUE);
        $ids['fields'][] = [
          'field_id' => $d['field']['id'] ?? NULL,
          'field_tag' => $d['field']['perstag'] ?? NULL,
          'field_title' => $d['field']['title'] ?? NULL,
        ];
      }
    }
    return $ids;
  }

  /**
   * Making relation of field to list.
   *
   * @param mixed $list_id
   *   List id for relationShip creation.
   * @param array $createdFields
   *   Fields to below to list id.
   *
   * @return bool
   *   True if relation made.
   */
  public function createRelation(mixed $list_id,
    array $createdFields): bool {

    if (!empty($list_id) && !empty($createdFields['fields'])) {
      foreach ($createdFields['fields'] as $field) {
        $data = [
          'field' => intval($field['field_id']),
          'relid' => intval($list_id),
        ];

        $curl = curl_init();

        curl_setopt_array($curl, [
          CURLOPT_URL => "{$this->tokens[0]['url']}/api/3/fieldRels",
          CURLOPT_RETURNTRANSFER => TRUE,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "POST",
          CURLOPT_POSTFIELDS => json_encode([
            'fieldRel' => $data,
          ]),
          CURLOPT_HTTPHEADER => [
            "Api-Token: {$this->tokens[0]['key']}",
            "accept: application/json",
            "content-type: application/json",
          ],
        ]);
        $response = curl_exec($curl);
        $err = curl_error($curl);
        curl_close($curl);
        if ($err) {
          return FALSE;
        }
        $d = json_decode($response, TRUE);
        return !empty($d['fieldRel']['id']);
      }
    }
    return FALSE;
  }

  /**
   * Grouping newly created fields.
   *
   * @param array $fieldsCreated
   *   Fields to group.
   * @param int $group
   *   Group id.
   *
   * @return bool
   *   True if a group made.
   */
  public function groupNewlyCreatedFields(array $fieldsCreated,
                                          int $group = 1): bool {
    if (!empty($fieldsCreated['fields']) && !empty($group)) {
      $flags = [];
      foreach ($fieldsCreated['fields'] as $value) {
        $curl = curl_init();

        curl_setopt_array($curl, [
          CURLOPT_URL => "{$this->tokens[0]['url']}/api/3/groupMembers",
          CURLOPT_RETURNTRANSFER => TRUE,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "POST",
          CURLOPT_POSTFIELDS => json_encode([
            'group_id' => $group,
            'rel_id' => $value['field_id'],
          ]),
          CURLOPT_HTTPHEADER => [
            "Api-Token: {$this->tokens[0]['key']}",
            "accept: application/json",
            "content-type: application/json",
          ],
        ]);

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        if ($err) {
          echo "cURL Error #:" . $err;
        }
        $d = json_decode($response, TRUE);
        if (!empty($d['groupMember']['id'])) {
          $flags[] = TRUE;
        }
      }
      return count($fieldsCreated['fields']) === count($flags);
    }
    return FALSE;
  }

}
